"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _react = _interopRequireDefault(require("react"));

var _server = require("react-dom/server");

var _prism = _interopRequireDefault(require("./utils/prism"));

var _escapeHtml = _interopRequireDefault(require("escape-html"));

var _normalizeHtml = _interopRequireDefault(require("./utils/normalizeHtml.js"));

var _htmlToPlain = _interopRequireDefault(require("./utils/htmlToPlain.js"));

var _selectionRange3 = _interopRequireDefault(require("./utils/selection-range.js"));

var _getIndent = require("./utils/getIndent");

var _constant = require("./utils/constant");

var _themes = require("./utils/themes");

var _languages = require("./utils/languages");

var _plugins = require("./utils/plugins");

var _getPrismSource = require("./getPrismSource");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Editor = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(Editor, _React$Component);

  function Editor(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "undoStack", []);

    _defineProperty(_assertThisInitialized(_this), "undoOffset", 0);

    _defineProperty(_assertThisInitialized(_this), "lastPos", 0);

    _defineProperty(_assertThisInitialized(_this), "undoTimestamp", 0);

    _defineProperty(_assertThisInitialized(_this), "composing", false);

    _defineProperty(_assertThisInitialized(_this), "onPaste", function (e) {
      e.preventDefault();
      var currentCursorPos = (0, _selectionRange3["default"])(_this.pre); // get text representation of clipboard

      var text = (e.originalEvent || e).clipboardData.getData("Text"); // insert text manually

      document.execCommand("insertHTML", false, (0, _escapeHtml["default"])(text));
      var newCursorPos = currentCursorPos.end + text.length;
      _this.selection = {
        start: newCursorPos,
        end: newCursorPos
      };

      var plain = _this.getPlain();

      _this.recordChange(plain, _this.selection);

      _this.updateContent(plain);

      _this.styleLineNumbers();
    });

    _defineProperty(_assertThisInitialized(_this), "addedDomNames", []);

    _defineProperty(_assertThisInitialized(_this), "deletePx", function (str) {
      if (str === void 0) {
        str = "";
      }

      return parseInt(str.split("px")[0]);
    });

    _this.state = {
      lineNumbersHeight: "20px",
      selection: undefined,
      codeData: "",
      content: "",
      id: ("" + Math.random()).split("0.")[1]
    };
    return _this;
  }

  var _proto = Editor.prototype;

  _proto.getLanguage = function getLanguage(language) {
    if (_languages.basicLanguages.includes(language)) {
      return language;
    } else {
      console.warn("Provided languages:" + _languages.basicLanguages);
      console.warn("Your input language is not support, use default language javascript");
      return "javascript";
    }
  };

  _proto.getTheme = function getTheme(theme) {
    if (_themes.basicThemes.includes(theme)) {
      return theme;
    } else {
      console.warn("Provided themes:" + _themes.basicThemes);
      console.warn("Your input theme is not support, use default theme");
      return "default";
    }
  };

  _proto.handleClick = function handleClick(evt) {
    this.undoTimestamp = 0;
    this.selection = (0, _selectionRange3["default"])(this.pre);
  };

  _proto.getPlain = function getPlain() {
    if (this._innerHTML === this.pre.innerHTML) {
      return this._plain;
    }

    var plain = (0, _htmlToPlain["default"])((0, _normalizeHtml["default"])(this.pre.innerHTML));
    this._innerHTML = this.pre.innerHTML;
    this._plain = plain;
    return this._plain;
  };

  _proto.recordChange = function recordChange(plain, selection) {
    if (plain === this.undoStack[this.undoStack.length - 1]) {
      return;
    }

    if (this.undoOffset > 0) {
      this.undoStack = this.undoStack.slice(0, -this.undoOffset);
      this.undoOffset = 0;
    }

    var timestamp = Date.now();
    var record = {
      plain: plain,
      selection: selection
    }; // Overwrite last record if threshold is not crossed

    if (timestamp - this.undoTimestamp < 3000) {
      this.undoStack[this.undoStack.length - 1] = record;
    } else {
      this.undoStack.push(record);

      if (this.undoStack.length > 50) {
        this.undoStack.shift();
      }
    }

    this.undoTimestamp = timestamp;
  };

  _proto.restoreStackState = function restoreStackState(offset) {
    console.log(this.undoStack);
    var _this$undoStack = this.undoStack[this.undoStack.length - 1 - offset],
        plain = _this$undoStack.plain,
        selection = _this$undoStack.selection;
    this.selection = selection;
    this.undoOffset = offset;
    this.updateContent(plain);
  };

  _proto.undo = function undo() {
    var offset = this.undoOffset + 1;

    if (offset >= this.undoStack.length) {
      return;
    }

    this.restoreStackState(offset);
  };

  _proto.redo = function redo() {
    var offset = this.undoOffset - 1;

    if (offset < 0) {
      return;
    }

    this.restoreStackState(offset);
  };

  _proto.handleKeyDown = function handleKeyDown(evt) {
    if (evt.keyCode === 9 && !this.ignoreTabKey) {
      document.execCommand("insertHTML", false, "  ");
      evt.preventDefault();
    } else if (evt.keyCode === 8) {
      // Backspace Key
      var _selectionRange = (0, _selectionRange3["default"])(this.pre),
          cursorPos = _selectionRange.start,
          cursorEndPos = _selectionRange.end;

      if (cursorPos !== cursorEndPos) {
        return; // Bail on selections
      }

      var deindent = (0, _getIndent.getDeindentLevel)(this.pre.innerText, cursorPos);

      if (deindent <= 0) {
        return; // Bail when deindent level defaults to 0
      } // Delete chars `deindent` times


      for (var i = 0; i < deindent; i++) {
        document.execCommand("delete", false);
      }

      evt.preventDefault();
    } else if (evt.keyCode === 13) {
      // Enter Key
      var _selectionRange2 = (0, _selectionRange3["default"])(this.pre),
          _cursorPos = _selectionRange2.start;

      var indentation = (0, _getIndent.getIndent)(this.pre.innerText, _cursorPos); // https://stackoverflow.com/questions/35585421
      // add a space and remove it. it works :/

      document.execCommand("insertHTML", false, "\n " + indentation);
      document.execCommand("delete", false);
      evt.preventDefault();
    } else if ( // Undo / Redo
    evt.keyCode === 90 && evt.metaKey !== evt.ctrlKey && !evt.altKey) {
      if (evt.shiftKey) {
        this.redo();
      } else {
        this.undo();
      }

      evt.preventDefault();
    }
  };

  _proto.handleKeyUp = function handleKeyUp(evt) {
    var keyupCode = evt.which;

    if (this.composing) {
      if (keyupCode === 13) {
        // finish inputting via IM.
        this.composing = false;
      } else {
        // now inputting words using IM.
        // must not update view.
        return;
      }
    }

    if (!this.code) {
      this.codeData = evt.target.innerText;
    } // if (this.emitEvents) {
    //     this.$emit("keyup", evt);
    // }


    if (evt.keyCode === 91 || // left cmd
    evt.keyCode === 93 || // right cmd
    evt.ctrlKey || evt.metaKey) {
      return;
    } // Enter key


    if (evt.keyCode === 13) {
      this.undoTimestamp = 0;
    }

    this.selection = (0, _selectionRange3["default"])(this.pre);

    if (!Object.values(_constant.FORBIDDEN_KEYS).includes(evt.keyCode)) {
      var plain = this.getPlain();
      this.recordChange(plain, this.selection);
      this.updateContent(plain);
    } else {
      this.undoTimestamp = 0;
    }
  };

  _proto.getContent = function getContent(codeData, language) {
    return (0, _prism["default"])(this.Prism, codeData || "", language);
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.pre.removeEventListener("paste", this.onPaste);
  };

  _proto.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
    if (this.props.code !== nextProps.code || this.props.language !== nextProps.language) {
      this.setPrismContent(nextProps.code, nextProps.language);
    }

    if (this.props.theme !== nextProps.theme) {
      this.setThemeStyle(nextProps.theme);
      this.setLineNumbersStyle(nextProps.theme);
      this.setState({}, this.styleLineNumbers);
    }

    return true;
  };

  _proto.componentDidMount = function componentDidMount() {
    var _this2 = this;

    // document.addEventListener('DOMContentLoaded',()=>{
    //     console.log('DOMContentLoaded')
    // })
    // const event = new CustomEvent("DOMContentLoaded", {
    //     detail: {
    //       hazcheeseburger: true
    //     }
    //   })
    // setInterval(() => {
    //     document.dispatchEvent(event)
    // }, 1000);
    var _this$props = this.props,
        theme = _this$props.theme,
        code = _this$props.code,
        language = _this$props.language;
    this.setThemeStyle(theme);
    this.setLineNumbersStyle(theme);
    this.setPrismScript();
    this.setPluginsScript(); //php等语言需要先引入这个

    this.setLanguageScript("markup-templating");
    this.Prism = window.Prism;
    this.setPrismContent(code, language);
    this.setState({}, function () {
      _this2.recordChange(_this2.getPlain());
    });
    this.undoTimestamp = 0; // Reset timestamp

    var $pre = this.pre;
    $pre.addEventListener("paste", this.onPaste);
    $pre.addEventListener("compositionstart", function () {
      _this2.composing = true;
    });
    $pre.addEventListener("compositionend", function () {
      // for canceling input.
      _this2.composing = false;
    });
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    var _this3 = this;

    this.addedDomNames.forEach(function (domName) {
      _this3[domName] && document.body.removeChild(_this3[domName]);
    });
  };

  _proto.updateContent = function updateContent(plain) {
    var _this$props2 = this.props,
        changeCode = _this$props2.changeCode,
        language = _this$props2.language;

    if (changeCode) {
      changeCode(plain);
    }

    this.setPrismContent(plain, language); // }, () => this.styleLineNumbers())
  };

  _proto.addElement = function addElement(type, name, content) {
    var domName = name + "-" + type + "-dom";

    if (document.querySelector("#" + domName)) {
      // if (this[domName] || window[domName]) {
      return; //script必须每次通过appendChild的方式才会重新执行，所以先要remove掉

      document.body.removeChild(this[domName]);
    } else {
      this.addedDomNames.push(domName);
    } // window[domName] = true


    this[domName] = document.createElement(type);
    this[domName].id = domName;
    this[domName].innerHTML = content;
    document.body.appendChild(this[domName]);
  };

  _proto.setPrismContent = function setPrismContent(code, language) {
    var _this4 = this;

    // debugger
    // this.Prism = null
    // this.setPrismScript()
    this.setLanguageScript(language); // const event = new CustomEvent("DOMContentLoaded", {
    //     detail: {
    //         hazcheeseburger: true
    //     }
    // })
    // document.dispatchEvent(event)
    // setInterval(() => {
    //
    // }, 1000);
    //

    this.setState({
      codeData: code || "",
      content: this.getContent(code, language)
    }, function () {
      // const container = this.pre.parentNode
      //去掉 toolbar，但是不能改container的className，否则每次keyUp光变都要消失
      // container.className = container.className.replace(/code-toolbar/g, '')
      //const toolbar = container.querySelector('.toolbar')
      // toolbar && container.removeChild(toolbar)
      //重新执行Prism的complete操作，linenumber,toobar等插件都在complete里挂载了回调方法
      _this4.Prism.hooks.run("complete", {
        language: language,
        code: code,
        element: _this4.pre.querySelector("code")
      }); // this.styleLineNumbers()

    });
  };

  _proto.setThemeStyle = function setThemeStyle(theme) {// this.addElement(
    //   "style",
    //   theme,
    //   addCssParent(`.module-theme-${theme}`, themesCss[theme])
    // );
  };

  _proto.setLineNumbersStyle = function setLineNumbersStyle(theme) {// this.addElement(
    //   "style",
    //   `${theme}-lineNumbers`,
    //   addCssParent(`.module-theme-${theme}`, lineNumbersCss)
    // );
  };

  _proto.setPrismScript = function setPrismScript() {// this.addElement("script", "prism", prismJs);
  };

  _proto.setPluginsScript = function setPluginsScript() {// Array.from(new Set(plugins.map((item) => item.value))).forEach((item) => {
    //   this.addElement("script", item, pluginsJs[item]);
    // });
  };

  _proto.setLanguageScript = function setLanguageScript(language) {// this.addElement("script", `${language}-language`, languagesJs[language]);
    // this.Prism.language = window.Prism.language
    // console.log(this.Prism)
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    if (this.selection) {
      (0, _selectionRange3["default"])(this.pre, this.selection);
    }
  } // getLineNumbers() {
  //     let totalLines = this.state.codeData.split(/\r\n|\n/).length;
  //     // TODO: Find a better way of doing this - ignore last line break (os spesific etc.)
  //     if (this.state.codeData.endsWith("\n")) {
  //         totalLines--;
  //     }
  //     const lineNumbers = []
  //     for (let i = 0; i < totalLines; i++) {
  //         lineNumbers.push(i)
  //     }
  //     return lineNumbers
  // }
  ;

  _proto.styleLineNumbers = function styleLineNumbers() {// clearTimeout(this.timeout)
    // if (this.props.lineNumber) {
    //     this.timeout = setTimeout(() => {
    //         const lineNumbers = this.getLineNumbers()
    //         const reactLineNumbers = lineNumbers.map((item, i) => <span key={i} />)
    //         const $editor = this.pre;
    //         const $code = $editor.querySelector('code')
    //         const editorStyles = window.getComputedStyle($editor);
    //         let $lineNumbers = $code.querySelector('.line-numbers-rows')
    //         if (!$lineNumbers) {
    //             $lineNumbers = document.createElement('div')
    //             $code.appendChild($lineNumbers)
    //             $lineNumbers.className = "line-numbers-rows"
    //             const btlr = "border-top-left-radius";
    //             const bblr = "border-bottom-left-radius";
    //             $lineNumbers.style[btlr] = editorStyles[btlr];
    //             $lineNumbers.style[bblr] = editorStyles[bblr];
    //             $editor.style[btlr] = 0;
    //             $editor.style[bblr] = 0;
    //             const stylesList = [
    //                 // "background-color",
    //                 "font-family",
    //                 "font-size",
    //                 "line-height",
    //                 // "padding-top",
    //                 // "padding-bottom",
    //             ];
    //             stylesList.forEach(style => {
    //                 $lineNumbers.style[style] = editorStyles[style];
    //             });
    //         }
    //         $lineNumbers.innerHTML = renderToString(reactLineNumbers)
    //         $editor.style['height'] = $lineNumbers.offsetHeight +
    //             // this.deletePx(editorStyles["padding-top"]) +
    //             // this.deletePx(editorStyles["padding-bottom"]) +
    //             // this.deletePx(editorStyles["border-top-width"]) +
    //             // this.deletePx(editorStyles["border-bottom-width"]) +
    //             'px'
    //     }, 20)
    // }
  };

  _proto.render = function render() {
    var _this5 = this;

    var _this$props3 = this.props,
        language = _this$props3.language,
        readOnly = _this$props3.readOnly,
        theme = _this$props3.theme,
        lineNumber = _this$props3.lineNumber,
        clipboard = _this$props3.clipboard,
        showLanguage = _this$props3.showLanguage;
    var _this$state = this.state,
        content = _this$state.content,
        lineNumbersHeight = _this$state.lineNumbersHeight,
        id = _this$state.id;
    return /*#__PURE__*/_react["default"].createElement("div", {
      className: "prism-editor-" + id + " module-theme-" + theme
    }, /*#__PURE__*/_react["default"].createElement("pre", {
      className: "language-" + language + " line-numbers copy-to-clipboard show-language",
      ref: function ref(_ref) {
        return _this5.pre = _ref;
      },
      style: {
        marginTop: 0
      },
      dangerouslySetInnerHTML: {
        __html: content
      },
      contentEditable: !readOnly,
      onKeyDown: this.handleKeyDown.bind(this),
      onKeyUp: this.handleKeyUp.bind(this),
      onClick: this.handleClick.bind(this),
      spellCheck: "false",
      autoCapitalize: "off",
      autoComplete: "off",
      autoCorrect: "off",
      "data-gramm": "false"
    }), /*#__PURE__*/_react["default"].createElement("style", null, "\n            .prism-editor-" + id + " .line-numbers-rows{\n                display:" + (lineNumber ? "block" : "none") + ";\n            }\n            .prism-editor-" + id + " .toolbar .toolbar-item:nth-child(1){\n                display:" + (showLanguage ? "inline-block" : "none") + ";\n            }\n            .prism-editor-" + id + " .toolbar .toolbar-item:nth-child(2){\n                display:" + (clipboard ? "inline-block" : "none") + ";\n            }\n            "));
  };

  return Editor;
}(_react["default"].Component);

var _default = Editor;
exports["default"] = _default;
module.exports = exports.default;